<?php
namespace App\Factory;

use App\Exception\Exception;
use Doctrine\DBAL\Types\StringType;

use function DI\string;

class PrintFactory {
    private $connection;
    private $util;

    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util = $util;
    }

    public function getBranchDetails($cdFilial, $cdloja){
        $sql = "
            SELECT IDLUGARMESA, IDPOSOBSPED, IDUTLCORTEPED
                FROM LOJA
            WHERE CDFILIAL = :CDFILIAL
                AND CDLOJA = :CDLOJA
        ";
        $params = [
            ':CDFILIAL' => $cdFilial,
            ':CDLOJA'   => $cdloja
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getTableDetails($cdFilial, $cdloja, $nrMesa){
        $sql = "
            SELECT M.CDFILIAL, M.CDLOJA, M.NRMESA, S.NMSALA
                FROM MESA M JOIN SALA S
                            ON M.CDFILIAL = S.CDFILIAL
                            AND M.CDSALA = S.CDSALA
            WHERE M.CDFILIAL = :CDFILIAL
                AND M.CDLOJA = :CDLOJA
                AND M.NRMESA = :NRMESA
        ";
        $params = [
            ':CDFILIAL' => $cdFilial,
            ':CDLOJA'   => $cdloja,
            ':NRMESA'   => $nrMesa
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getSellerDetails($cdVendedor){
        $sql = "
            SELECT NMFANVEN
                FROM VENDEDOR
            WHERE CDVENDEDOR = :CDVENDEDOR
        ";
        $params = [
            ':CDVENDEDOR' => $cdVendedor
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getGeneralParams($cdFilial){
        $sql = "
        SELECT CDFILIAL, NRATRAPADRAO, IDIMPCOMPACTA, 
               IDPERVENDCONTSP, IDCOMPPISCOFINS
            FROM PARAVEND
        WHERE CDFILIAL = :CDFILIAL
        ";
        $params = [
            ':CDFILIAL' => $cdFilial
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getPrinterByProd($cdFilial, $cdloja, $cdProduto){
        $sql = "
            SELECT A.NRSEQIMPRPROD AS A_NRSEQIMPRPROD, A.NRSEQIMPRLOJA AS A_NRSEQIMPRLOJA, A.NMIMPRLOJA AS A_NMIMPRLOJA,
                A.CDPORTAIMPR AS A_CDPORTAIMPR, A.IDMODEIMPRES AS A_IDMODEIMPRES, A.DSENDPORTA AS A_DSENDPORTA,
                A.DSIPIMPR AS A_DSIPIMPR, A.DSIPPONTE AS A_DSIPPONTE,

                B.NRSEQIMPRPROD2 AS B_NRSEQIMPRPROD2, B.NRSEQIMPRLOJA AS B_NRSEQIMPRLOJA, B.NMIMPRLOJA AS B_NMIMPRLOJA,
                B.CDPORTAIMPR AS B_CDPORTAIMPR, B.IDMODEIMPRES AS B_IDMODEIMPRES, B.DSENDPORTA AS B_DSENDPORTA,
                B.DSIPIMPR AS B_DSIPIMPR, B.DSIPPONTE AS B_DSIPPONTE,

                C.NRSEQIMPRPUXA AS C_NRSEQIMPRPUXA, C.NRSEQIMPRLOJA AS C_NRSEQIMPRLOJA, C.NMIMPRLOJA AS C_NMIMPRLOJA,
                C.CDPORTAIMPR AS C_CDPORTAIMPR, C.IDMODEIMPRES AS C_IDMODEIMPRES, C.DSENDPORTA AS C_DSENDPORTA,
                C.DSIPIMPR AS C_DSIPIMPR, C.DSIPPONTE AS C_DSIPPONTE

            FROM

            (SELECT P.NRSEQIMPRPROD, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR,
                    I.IDMODEIMPRES, M.DSENDPORTA, L.DSIPIMPR, L.DSIPPONTE
            FROM PRODLOJA P JOIN IMPRLOJA L
                                ON P.CDFILIAL = L.CDFILIAL
                                AND P.CDLOJA = L.CDLOJA
                                AND P.NRSEQIMPRPROD = L.NRSEQIMPRLOJA

                            JOIN IMPRESSORA I
                                ON L.CDIMPRESSORA = I.CDIMPRESSORA

                        LEFT JOIN MAPIMPRLOJA M
                                ON L.CDFILIAL = M.CDFILIAL
                                AND L.CDLOJA = M.CDLOJA
                                AND L.CDPORTAIMPR = M.CDPORTAIMPR
            WHERE P.CDFILIAL = :CDFILIAL
                AND P.CDLOJA = :CDLOJA
                AND P.CDPRODUTO = :CDPRODUTO) A

            FULL JOIN

            (SELECT P.NRSEQIMPRPROD2, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR,
                    I.IDMODEIMPRES, M.DSENDPORTA, L.DSIPIMPR, L.DSIPPONTE
            FROM PRODLOJA P JOIN IMPRLOJA L
                                ON P.CDFILIAL = L.CDFILIAL
                                AND P.CDLOJA = L.CDLOJA
                                AND P.NRSEQIMPRPROD2 = L.NRSEQIMPRLOJA

                            JOIN IMPRESSORA I
                                ON L.CDIMPRESSORA = I.CDIMPRESSORA

                        LEFT JOIN MAPIMPRLOJA M
                                ON L.CDFILIAL = M.CDFILIAL
                                AND L.CDLOJA = M.CDLOJA
                                AND L.CDPORTAIMPR = M.CDPORTAIMPR
            WHERE P.CDFILIAL = :CDFILIAL
                AND P.CDLOJA = :CDLOJA
                AND P.CDPRODUTO = :CDPRODUTO) B

            ON 'A' = 'A' FULL JOIN

            (SELECT P.NRSEQIMPRPUXA, L.NRSEQIMPRLOJA, L.NMIMPRLOJA, L.CDPORTAIMPR,
                    I.IDMODEIMPRES, M.DSENDPORTA, L.DSIPIMPR, L.DSIPPONTE
            FROM PRODLOJA P JOIN IMPRLOJA L
                                ON P.CDFILIAL = L.CDFILIAL
                                AND P.CDLOJA = L.CDLOJA
                                AND P.NRSEQIMPRPUXA = L.NRSEQIMPRLOJA

                            JOIN IMPRESSORA I
                                ON L.CDIMPRESSORA = I.CDIMPRESSORA

                        LEFT JOIN MAPIMPRLOJA M
                                ON L.CDFILIAL = M.CDFILIAL
                                AND L.CDLOJA = M.CDLOJA
                                AND L.CDPORTAIMPR = M.CDPORTAIMPR
            WHERE P.CDFILIAL = :CDFILIAL
                AND P.CDLOJA = :CDLOJA
                AND P.CDPRODUTO = :CDPRODUTO) C

            ON 'A' = 'A'
        ";
        $params = [
            ':CDFILIAL'  => $cdFilial,
            ':CDLOJA'    => $cdloja,
            ':CDPRODUTO' => $cdProduto
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getPrinterDetails($CDFILIAL, $CDLOJA, $NRSEQIMPRLOJA){
        $sql = "
            SELECT P.NRSEQIMPRLOJA, I.IDMODEIMPRES, P.DSIPIMPR, P.DSIPPONTE,
                   P.CDPORTAIMPR, M.DSENDPORTA, P.NMIMPRLOJA
                FROM IMPRLOJA P
            JOIN IMPRESSORA I ON I.CDIMPRESSORA = P.CDIMPRESSORA
            LEFT JOIN MAPIMPRLOJA M ON P.CDFILIAL = M.CDFILIAL
                                    AND P.CDLOJA = M.CDLOJA
                                    AND P.CDPORTAIMPR = M.CDPORTAIMPR
            WHERE P.CDFILIAL      = :CDFILIAL
                AND P.CDLOJA        = :CDLOJA
                AND P.NRSEQIMPRLOJA = :NRSEQIMPRLOJA

        ";
        $params = [
            ':CDFILIAL'      => $CDFILIAL,
            ':CDLOJA'        => $CDLOJA,
            ':NRSEQIMPRLOJA' => $NRSEQIMPRLOJA
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getPrinterByEnvironment($CDFILIAL, $CDLOJA, $produtoAtual){
        $sql = "
        SELECT I.NRSEQIMPRLOJA, S.IDMODEIMPRES, I.DSIPIMPR, I.DSIPPONTE,
               I.CDPORTAIMPR, M.DSENDPORTA, I.NMIMPRLOJA
            FROM PRODAMBIENTE P JOIN IMPRLOJA I
                                    ON P.CDFILIAL = I.CDFILIAL
                                    AND P.CDLOJA = I.CDLOJA
                                    AND P.NRSEQIMPRLOJA = I.NRSEQIMPRLOJA
                                    JOIN IMPRESSORA S
                                    ON I.CDIMPRESSORA = S.CDIMPRESSORA
                            LEFT JOIN MAPIMPRLOJA M
                                    ON I.CDFILIAL = M.CDFILIAL
                                    AND I.CDLOJA = M.CDLOJA
                                    AND I.CDPORTAIMPR = M.CDPORTAIMPR
            WHERE P.CDFILIAL = :CDFILIAL
            AND P.CDLOJA = :CDLOJA
            AND P.CDPRODUTO = :CDPRODUTO
            AND P.NRCONFTELA = :NRCONFTELA
            AND P.DTINIVIGENCIA = :DTINIVIGENCIA
            AND P.CDAMBIENTE = :CDAMBIENTE
        ";
        $params = [
            'CDFILIAL'      => $CDFILIAL,
            'CDLOJA'        => $CDLOJA,
            'CDPRODUTO'     => $produtoAtual['CDPRODUTO'],
            'NRCONFTELA'    => $produtoAtual['NRCONFTELA'],
            'DTINIVIGENCIA' => new \DateTime($produtoAtual['DTINIVIGENCIA']),
            'CDAMBIENTE'    => $produtoAtual['CDSALA']
        ];
        $types = [
            'DTINIVIGENCIA' => \Doctrine\DBAL\Types\Type::DATETIME
        ];

        return $this->connection->fetchAssoc($sql, $params, $types);
    }

    public function getPrinterData($CDFILIAL, $CDCAIXA){
        $sql = "
            SELECT P.NRSEQIMPRLOJA, I.IDMODEIMPRES, P.CDPORTAIMPR, L.CDLOJA,
                    UPPER(L.NMLOJA) AS NMLOJA, UPPER(C.NMCAIXA) AS NMCAIXA,
                    P.DSIPIMPR, P.DSIPPONTE, C.CDCAIXA, C.NMCAIXA, P.NMIMPRLOJA
            FROM CAIXA C JOIN IMPRLOJA P
                            ON C.CDFILIAL = P.CDFILIAL
                        AND C.CDLOJA = P.CDLOJA
                        AND C.NRSEQIMPRLOJA3 = P.NRSEQIMPRLOJA
                        JOIN IMPRESSORA I
                            ON P.CDIMPRESSORA = I.CDIMPRESSORA
                        JOIN LOJA L
                            ON L.CDLOJA = C.CDLOJA
                        AND L.CDFILIAL = C.CDFILIAL
            WHERE C.CDFILIAL = :CDFILIAL
            AND C.CDCAIXA = :CDCAIXA
        ";
        $params = [
            'CDFILIAL' => $CDFILIAL,
            'CDCAIXA' => $CDCAIXA
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getNameByPosition($produtos, $posicoes){
        $sql = "
            SELECT NRLUGARMESA, DSCONSUMIDOR
                FROM POSVENDAREST
                    WHERE CDFILIAL = :CDFILIAL
                     AND NRVENDAREST = :NRVENDAREST
                     AND NRLUGARMESA IN (:NRLUGARMESA)
                     AND DSCONSUMIDOR IS NOT NULL
        ";
        $params = [
            'CDFILIAL' => $produtos['CDFILIAL'],
            'NRVENDAREST' => $produtos['NRVENDAREST'],
            'NRLUGARMESA' => $posicoes
        ];
        $type = array(
            'NRLUGARMESA' => \Doctrine\DBAL\Connection::PARAM_STR_ARRAY
        );

        return $this->connection->fetchAssoc($sql, $params, $type);
    }

}