<?php

namespace App\Service;

use App\Exception;

class ProductService
{

    private $productFactory;
    private $orderService;
    private $util;

    public function __construct(
        \App\Factory\ProductFactory $productFactory,
        \App\Service\OrderService $orderService,
        \App\Util\Util $util
    ) {
        $this->productFactory = $productFactory;
        $this->orderService = $orderService;
        $this->util = $util;
    }

    public function processProducts($items, $orderType, $cdFilial, $nrVendarest, $nrComanda, $cdLoja, $idControProd, $sellerPos, $cdvendedor, $pagerId)
    {
        $orderItemsSeq = $this->util->geraCodigo("ITCOMANDAVEN$cdFilial", 1, 10);
        //$itComandaVenSeq = $this->productFactory->getNrProdComVen($nrVendarest, $nrComanda);
        $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
        $NRSEQPRODCOM = $NRPRODCOMVEN;
        //if (!empty($itComandaVenSeq['NRPRODCOMVEN']))
        //    $NRPRODCOMVEN = $itComandaVenSeq['NRPRODCOMVEN'];
        //    $NRSEQPRODCOM = $itComandaVenSeq['NRPRODCOMVEN'];

        $simpleItens = [];
        foreach ($items as $item) {
            if (empty($item['price']) || ($item["price"] < 0.01))
                throw new \Exception("O produto: {$item['code']} - {$item['name']} não atende o preço mínimo por item (R$ 0,01).");
            $productDetails = $this->productFactory->getProduct($item['code']);
            if (empty($item['code']))
                throw new \Exception("O produto: {$item['code']} - {$item['name']} não possui código de integração parametrizado.");
            if (empty($productDetails))
                throw new \Exception("O produto: {$item['code']} - {$item['name']} não encontrado. Verifique a parametrização.");

            $productDetails = array_merge($productDetails, $item);
            $productDetails["TXPRODCOMVEN"] = " ";
            $hasComplement = false;
            if (!empty($item["options"]) && count($item["options"]) > 0) {
                foreach ($productDetails["options"] as $obs) {
                    if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                        $obsTitle = $this->getObservationName($obs);
                        for ($i = 0; $i < $obs["quantity"]; $i++)
                            $productDetails["TXPRODCOMVEN"] .= $obsTitle . "; ";
                    }
                    if (($obs['price'] > 0.01) && (!$hasComplement))
                        $hasComplement = true;
                }
            }
            $CDPRODPROMO = null;
            if ((!empty($item["options"])) && ($hasComplement))
                $CDPRODPROMO = $productDetails["CDPRODUTO"];
            $notes = "";
            if (!empty($item['notes']) && count($item['notes']) > 0) {
                foreach ($item['notes'] as $note) {
                    $notes .= $note . "; ";
                }
            }
            $productDetails["TXPRODCOMVEN"] .= empty($productDetails["notes"]) ? "" : implode(",", $productDetails["notes"]) . "; ";
            $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
            $NRSEQPRODCOM = $NRPRODCOMVEN;
            $productReturn = array(
                "CDFILIAL"          => $cdFilial,
                "NRVENDAREST"       => $nrVendarest,
                "NRCOMANDA"         => $nrComanda,
                "NRPRODCOMVEN"      => $this->util->toRight($NRPRODCOMVEN, 6),
                "CDPRODUTO"         => $productDetails["CDPRODUTO"],
                "QTPRODCOMVEN"      => $item['quantity'],
                "VRPRECCOMVEN"      => $this->util->toFloat($item['price']),
                "TXPRODCOMVEN"      => $productDetails["TXPRODCOMVEN"],
                "IDSTPRCOMVEN"      => $idControProd === "S" ? "1" : "3",
                "VRDESCCOMVEN"      => 0.00,
                "NRLUGARMESA"       => "01",
                "IDPRODIMPFIS"      => "N",
                "CDLOJA"            => $cdLoja,
                "VRACRCOMVEN"       => 0.00,
                "NRSEQPRODCOM"      => null,
                "NRSEQPRODCUP"      => $orderItemsSeq,
                "VRPRECCLCOMVEN"    => 0.00,
                "CDCAIXACOLETOR"    => $sellerPos,
                "CDPRODPROMOCAO"    => null,
                "CDVENDEDOR"        => $cdvendedor,
                "NRINSCRCONS"       => null,
                "DSOBSPEDDIGCMD"    => $notes,
                "CDSENHAPED"        => $pagerId,
                "ITEM_OBSERVATION"  => [],
                "IDOBSERVACAOAD"    => 'N',
                "IDPRODEXT"      => $item['id']
            );

            if (!empty($productDetails["options"]) && count($productDetails["options"]) > 0) {
                foreach ($productDetails["options"] as $obs) {
                    if ($obs['price'] >= 0.01) {

                        if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                            $observationDetails = $this->productFactory->getObservation($obs['code'], true);
                        } else {
                            $observationDetails = $this->productFactory->getProduct($obs['code']);
                        }

                        if (empty($observationDetails))
                            throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                        for ($y = 0; $y < $obs['quantity']; $y++) {
                            $NRPRODCOMVEN = ($this->util->geraCodigo("ITCOMANDAVEN{$cdFilial}{$nrComanda}", 1, 6) - 1);
                            $NRSEQPRODCOM = $NRPRODCOMVEN;
                            $observationDetails["NRPRODCOMVEN"] = str_pad((string) ($NRPRODCOMVEN), 6, '0', STR_PAD_LEFT);


                            $obsReturn = array(
                                "CDFILIAL"       => $cdFilial,
                                "NRVENDAREST"    => $nrVendarest,
                                "NRCOMANDA"      => $nrComanda,
                                "NRPRODCOMVEN"   => $observationDetails["NRPRODCOMVEN"],
                                "CDPRODUTO"      => $observationDetails["CDPRODUTO"],
                                "QTPRODCOMVEN"   => 1.00,
                                "VRPRECCOMVEN"   => $this->util->toFloat($obs['price']),
                                "TXPRODCOMVEN"   => " ",
                                "IDSTPRCOMVEN"   => $idControProd === "S" ? "1" : "3",
                                "VRDESCCOMVEN"   => 0.00,
                                "NRLUGARMESA"    => "01",
                                "IDPRODIMPFIS"   => "N",
                                "CDLOJA"         => $cdLoja,
                                "VRACRCOMVEN"    => 0.00,
                                "NRSEQPRODCOM"   => $this->formatNrSeqProdCom($NRSEQPRODCOM),
                                "NRSEQPRODCUP"   => $orderItemsSeq,
                                "VRPRECCLCOMVEN" => 0.00,
                                "CDCAIXACOLETOR" => $sellerPos,
                                "CDPRODPROMOCAO" => $CDPRODPROMO,
                                "CDVENDEDOR"     => $cdvendedor,
                                "NRINSCRCONS"    => null,
                                "DSOBSPEDDIGCMD" => '',
                                "CDSENHAPED"     => $pagerId,
                                "IDOBSERVACAOAD" => 'S',
                                "NRSEQEXTPROD"   => $obs["id"],
                                "IDPRODEXT"      => $item['id']
                            );

                            array_push($simpleItens, $obsReturn);
                        }

                        if (strlen($obs['code']) === 5 && substr($obs['code'], 2, 1) === '|') {
                            array_push($productReturn['ITEM_OBSERVATION'], [
                                'CDFILIAL'        => $cdFilial,
                                'NRVENDAREST'     => $nrVendarest,
                                'NRCOMANDA'       => $nrComanda,
                                'NRPRODCOMVEN'    => $this->util->toRight($NRPRODCOMVEN, 6),
                                'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                                'CDOCORR'         => $observationDetails['CDOCORR'],
                                'NRPRODCOMVENOBS' => null, //$observationDetails["NRPRODCOMVEN"],  
                                'QTPRODCOMVENOBS' => $obs['quantity']
                            ]);
                        }
                    } else {
                        $observationDetails = $this->productFactory->getObservation($obs['code']);

                        if (empty($observationDetails))
                            throw new \Exception("Não foi possivel encontrar o complemento de código: {$obs['code']}");

                        array_push($productReturn['ITEM_OBSERVATION'], [
                            'CDFILIAL'        => $cdFilial,
                            'NRVENDAREST'     => $nrVendarest,
                            'NRCOMANDA'       => $nrComanda,
                            'NRPRODCOMVEN'    => $this->util->toRight($NRPRODCOMVEN, 6),
                            'CDGRPOCOR'       => $observationDetails['CDGRPOCOR'],
                            'CDOCORR'         => $observationDetails['CDOCORR'],
                            'NRPRODCOMVENOBS' => null,
                            'QTPRODCOMVENOBS' => 1
                        ]);
                    }
                }
            }
            array_push($simpleItens, $productReturn);
        }
        return $simpleItens;
    }

    private function getObservationName($observation)
    {

        if (empty($observation["code"]))
            throw new \Exception("A observação: " . $observation["name"] . ", não possui código de integração parametrizado.");

        $isAcr = false;
        if ($observation['price'] >= 0.01) {
            $isAcr = true;
        }

        $observationDetails = $this->productFactory->getObservation($observation["code"], $isAcr);

        if (empty($observationDetails))
            throw new \Exception("Não foi encontrada ocorrência para o código de integração: " . $observation["code"] . " Complemento - " . $observation["name"], 120);

        return $observationDetails["DSOCORR"];
    }

    private function formatNrSeqProdCom($seq)
    {
        return str_pad((string) ((int) $seq), 3, '0', STR_PAD_LEFT);
    }

    public function insertProduct($item, $generalParams)
    {
        return $this->productFactory->insertProduct($item, $generalParams);
    }

    public function insertObservation($item, $generalParams, $orderData)
    {
        return $this->productFactory->insertProduct($item, $generalParams, $orderData);
    }

    public function ProductsForShipping($cdFilial, $produtsPartner)
    {
        
        $products = $this->productFactory->ProductsForShipping($cdFilial);
        $observations = $this->productFactory->observationsForShipping($cdFilial);
        $products =array_merge($products, $observations);
        $arrayProducts = [];

        foreach ($products as $product) {
            $isExist = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O PRODUTO JÁ FOI CADASTRADO NA ABRAHAO
                foreach ($produtsPartner as $prodExt) {
                    if ($product['CDARVPROD'] == $prodExt['code']) {
                        $isExist = true;
                    }
                }
            }

            if (!$isExist) {
                $arrayProd = [
                    'code'         => $product['CDARVPROD'],
                    'name'         => $product['NMPRODUTO'],
                    'price'        => !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : $this->util->toFloat($product['PRECOGER']),
                ];
                array_push($arrayProducts, $arrayProd);
                $contente = date("d:m:Y H:i:s") . " - " . "Cadastrando produto {$product['CDARVPROD']} - {$product['NMPRODUTO']} \n";
                $this->util->writeLog('sendProducts_execution', '', $contente);
            }
        }
        return $arrayProducts;
    }

    public function ProductsToUpdating($cdFilial, $produtsPartner)
    {

        $products = $this->productFactory->ProductsForShipping($cdFilial);
        $update = [];

        foreach ($products as $product) {
            $isDiff = false;

            if (!empty($produtsPartner)) {

                //  VALIDA SE O VALOR DO PRODUTO É DIFERENTE DO SISTEMA TEKNISA
                foreach ($produtsPartner as $prodExt) {
                    if ($product['CDARVPROD'] == $prodExt['code']) {
                        $precoFili = !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : 0;
                        $precoGer = !empty($product['PRECOGER']) ? $this->util->toFloat($product['PRECOGER']) : 0;
                        $price = !empty($prodExt['price']) ? $this->util->toFloat($prodExt['price']) : 0;
     
                        if ((empty($precoFili)) && ($precoGer != $price)) {
                            $isDiff = true;
                        } elseif ((!empty($precoFili)) && ($precoFili != $price)) {
                            $isDiff = true;
                        }
                    }
                }
            }
            
            if(($isDiff)){
                $prodUpdate = [
                    'code'    => $product['CDARVPROD'], 
                    'request' => [
                        'name'  => $product['NMPRODUTO'],
                        'price'   => !empty($product['PRECOFILI']) ? $this->util->toFloat($product['PRECOFILI']) : $this->util->toFloat($product['PRECOGER'])
                    ]
                ];
                array_push($update, $prodUpdate);
                $contente = date("d:m:Y H:i:s") . "Atualização de preço para o produto {$product['CDARVPROD']} - {$product['NMPRODUTO']} \n";
                $this->util->writeLog('sendProducts_execution', '', $contente);
            }
        }
        return $update;
    }
}
